function geom(varargin)

%GEOM - Some utility functions for Osmo Hakosalo's EISCAT package.
%
% The following subcommands are available:
%
%   geom info    - print info about the state of the package
%   geom help    - print available commands
%   geom reset   - reset global state of the package
%   geom format short|long - affects geom info printing format
%
% See also pointrrange, pointrheight, pointgeographic, changersite.   

% 25-Mar-2002 Jm: geom splash added.
% 26-Mar-2004 Jm: ESR coordinates changed, see private/init_ogeo_sites.m

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
global OGEOM

%version = '1.1 25-Sep-02';
version = '1.2 26-Mar-04';

if isempty(OGEOM)
    init(version);
end

% Get the subcommand

try
    [cmd,msg,par] = get_command(varargin);
catch
    msg = 'Illegal format';
end                        

% Execute the subcommand

if isempty(msg)                
    switch cmd
        case 'help'
            print_help;
        case 'info'
            print_info;    
        case 'format'
            msg = set_format(par{1});            
        case 'reset'
            reset_us;
        case 'splash'
            show_splash;    
        otherwise
            ;
    end
end
disp(msg)
return
%end ogeo


function init(version)
%==============================================================
%
%==============================================================
global OGEOM
define_commands;
OGEOM.version = version;
OGEOM.format = 'short';  % 'short' or 'long'
OGEOM.rsite = 'T';
[gcT,gcS,gcK,gcESR,sgTT,sgKK,sgSS,sgESR,TX] = init_ogeo_sites;
if TX(4) == 90
    OGEOM.txpol = 'left';
elseif TX(4) == -90
    OGEOM.txpol = 'right';
else
    OGEOM.txpol = '???';
end        
return
%end init

function msg = set_format(s)
%==============================================================
%
%==============================================================
global OGEOM
msg = '';
if isempty(s)
    msg = OGEOM.format;
    return
end
s = lower(s);
if strncmp(s,'s',1)
    OGEOM.format = 'short';
    msg = OGEOM.format;
elseif strncmp(s,'l',1);
    OGEOM.format = 'long';
    msg = OGEOM.format;
else
    msg = 'Illegal parameter for format -- must be "long"  or "short"';                        
end
return
%end set_format


function site = rsite(site)
%==============================================================
%
%==============================================================
global OGEOM
if nargin == 1
    OGEOM.rsite = site;
elseif nargin == 0
    site = OGEOM.rsite;
end    
return
%end rsite


function msg = print_help
%==============================================================
%
%==============================================================
global OGEOM
msg = '';

fprintf(1,'EISCAT GEOMETRY -- version %s\n', OGEOM.version);
fprintf(1,'  Available commands:\n');
fprintf(1,'     pointrrange Az El R\n');
fprintf(1,'     pointrheight Az El H\n');
fprintf(1,'     pointgeographic Lat Lon H\n');
fprintf(1,'     changersite K|S|T|E\n');
fprintf(1,'     setpolarizer\n');
fprintf(1,'     geom format long|short\n');
fprintf(1,'     geom reset\n');
fprintf(1,'     geom info\n');
fprintf(1,'     geom help\n');
return
%end print_help

function msg = show_splash
%==============================================================
%
%==============================================================
global OGEOM
msg = '';
fprintf(1,'  EISCAT GEOMETRY -- Version %s  (Help: "geom help")\n\n', OGEOM.version);
%end show_splash


function define_commands
%==============================================================
% This defines the subcommands of the geom function
%==============================================================
global OGEOM

% All legal commands must be defined in the following table

commandtable = cell2struct({
    %--------------------------------------
    % NAME               TYPE
    %--------------------------------------
    'format',            'S';
    'reset',             '-';
    'help',              '-';   
    'info',              '-';
    'splash',            '-';
    'nop',               '-';
    % ------------------------------------
},{'name','type'},2);

% Split the command table into two lists for command name matcher

for k = 1:length(commandtable)
    cname{k} = commandtable(k).name;
    ctype{k} = commandtable(k).type;
end
OGEOM.cname = cname;
OGEOM.ctype = ctype;
return
%end define_commands


function msg = print_info
%==============================================================
%
%==============================================================
global OGEOM
fprintf(1,'EISCAT GEOM -- version %s.\n', OGEOM.version);
fprintf(1,'  TX:       %s-handed circular\n',OGEOM.txpol)
fprintf(1,'  Refsite:  %s\n',rsite);
%end print_info


function msg = reset_us
%==============================================================
%
%==============================================================
clear global OGEOM
geom nop;
%end reset




function [cmd,msg,param] = get_command(pars)
%==============================================================
% Read command from command line, return command name and
% parameters. If a valid command with correct number of
% parameters is returned, msg == '', else msg contains an
% error message.
%==============================================================
global OGEOM
msg = '';
cmd = '';
param = {};

if nargin > 0
    % Concatenate the arguments into a single string

    s = '';
    for k = 1:length(pars)
        if ischar(pars{k})
            s = [s ' ' pars{k}];
        else
            s = [s ' ' num2str(pars{k})];
        end                        
    end
else 
    % Read input from command line

    s = input('geo% ', 's');    
end    

if isempty(s) 
    cmd = 'nop';
    msg = '';
    return;
end

% First token on command line must be the command name, match it

[cmd,str] = strtok(s);
K = matcher(cmd,OGEOM.cname);

if length(K) == 0
    % No match

    cmd = '';
    msg = ['Illegal command (' cmd ')'];
    
elseif length(K) > 1
    % More than one match
    
    cmd = '';
    msg = [' cmd ' -- ambiguous'];
else
    % Unique match, now find the command parameters   
    
    str = strip(str);
    
    cmd = OGEOM.cname{K};
    if  strcmp(OGEOM.ctype{K},'-')
        str = '';
    elseif strcmp(OGEOM.ctype{K},'NNN')        
        p = str2num(str);
        if length(p) ~= 3
            msg = 'Illegal format';
        else
            param{1} = p(1);
            param{2} = p(2);
            param{3} = p(3);
        end
    elseif strcmp(OGEOM.ctype{K},'S')        
        param{1} = str;
    else
        error([mfilename ': Illegal parameter type'])
    end        
    
end
str = strip(str);

return
%get_command


function K = matcher(name, namelist)
%================================================================
%K = MATCHER(NAME, NAMELIST)
%
% Return the index of NAME in NAMELIST. The NAME can be appreviated,
% and the case is not significant. NAMELIST can be cell array
% or a character array. If no match is found, K is returned empty.
% If a unique match is found, K is the index of the NAME in NAMELIST.
% If there are several matches, K is a vector of indexes of the
% matching names. It is assumed that there are do dublicates in words
% (taking into account that case is ignored). The NAMELIST should be
% in all lowercase.
%
% In case of multiple match where one is an exact match, the
% exact match is returned, and the situation is _not_
% reported as being ambiguous.
%
% 27-Oct-1999 Jm
%================================================================
name = lower(name);
K = strmatch(name,namelist);
if length(K) >= 2
    % Got several matches. We need to check if one is an exact
    % match. It is sufficient here to check the lengths only.
    k = [];
    if iscellstr(namelist) 
        for n = 1:length(K)
            fullname = namelist{K(n)};
            if length(name) == length(fullname);
                k = K(n);
                break;
            end                
        end        
    else
        for n = 1:length(K)
            fullname = deblank(namelist(K(n),:));
            if length(name) == length(fullname);
                k = K(n);
                break;
            end                
        end        
    end
    if ~isempty(k)  % got a unique match
        K = k;
    end                        
end    
%end matcher


function str = strip(str)
%================================================================= 
% Remove leading and trailing blanks.
%=================================================================
str = str(end:-1:1);
str = deblank(str);
str = str(end:-1:1);
str = deblank(str);

