function geom(varargin)

%ogeo - command line interface to OH's  EISCAT geometry package
%
%   This wrapper can be used in
%
%   (1) in single-command mode, for example
%
%       >> geo pointrrange 180 75 300
%       >> geo('pointrrange',180,75,300)
%   or
%
%   (2) in interactive mode, under its onw command processing loop.
%   The loop is entered by calling geo without any arguments
%
%       >> geo  ==> Interactive mode
%
%   In the "interactive" use, the command format is
%
%       geo% pointrrange 180 75 300      
%       geo% help
%
% See also ogeometry.m
%
% 29-May-2001 Jm
% 22-Mar-2002 Jm
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
global OGEOM
msg = '';
if ( nargin == 0 )
    interactive = 1;
    fprintf(1,'\nGEO in interactive mode -- Type "h" for HELP\n\n')
else
    interactive = 0;
end        

if isempty(OGEOM)
    msg = init;
    rsite('T');
end

done = 0;
while ~done
    msg = '';
    if interactive
        [cmd,msg,par] = get_command;
    else
        done = 1;
        [cmd,msg,par] = get_command(varargin);
    end                
    switch cmd
        case 'quit'
            done = '1';
        case 'help'
            print_help(interactive);
        case 'format'
            msg = set_format(par{1});            
        case 'reset'
            reset_us;
        case 'pointrrange'
            [msg,G] = ogeometry('PointReferenceRange',...
                                 par{1},par{2},par{3},rsite);
            if isempty(msg)
                print(G);
            end                
        case 'pointrheight'
            [msg,G] = ogeometry('PointReferenceHeight',...
                                 par{1},par{2},par{3},rsite);
            if isempty(msg)
                print(G);
            end                
        case 'pointgeographic'
            [msg,G] = ogeometry('PointGeographic',...
                                 par{1},par{2},par{3},rsite);
            if isempty(msg)
                print(G);
            end                
        case 'changersite'            
            [msg,site] = ogeometry('ChangeReferenceSite',...
                                    par{1});
            if isempty(msg)
                rsite(site);
                fprintf(1,'%s\n',site)
            end                
        otherwise
            ;
    end;%switch
    disp(msg)
end;%while    
return
%end ogeo


%==============================================================
%
%==============================================================
function msg = set_format(s)
    msg = '';
    global OGEOM
    if isempty(s)
        msg = OGEOM.format;
        return
    end
    s = lower(s);
    if strncmp(s,'s',1)
        OGEOM.format = 'short';
        msg = OGEOM.format;
    elseif strncmp(s,'l',1);
        OGEOM.format = 'long';
        msg = OGEOM.format;
    else
        msg = 'Illegal parameter for format -- must be "long"  or "short"';                        
    end
return
%end set_format


function site = rsite(site)
%==============================================================
%
%==============================================================
global OGEOM
if nargin == 1
    OGEOM.rsite = site;
elseif nargin == 0
    site = OGEOM.rsite;
end    
return
%end rsite


function msg = print_help(interactive)
%==============================================================
%
%==============================================================
global OGEOM
msg = '';
if interactive
    fprintf(1,'Available commands\n');
else
    fprintf(1,'Available %s subcommands\n',mfilename);
end        
fprintf(1,'  pointrrange  Az El R\n');
fprintf(1,'  pointrheight  Az El H\n');
fprintf(1,'  pointgeographic  Lat Lon H\n');
fprintf(1,'  changersite  K|S|T|L|E\n');
fprintf(1,'  format  Long|Short\n');
fprintf(1,'  reset\n');
fprintf(1,'  quit\n');
return
%end print_help


function msg = init
%==============================================================
%
%==============================================================
global OGEOM
msg = '';
define_commands
OGEOM.format = 'short';  % 'short' or 'long'
return
%end init


function msg = reset_us
%==============================================================
%
%==============================================================
global OGEOM
    msg = '';
    OGEOM.format = 'short';  % 'short' or 'long'
    OGEOM.rsite = 'T';
%end reset



function define_commands
%==============================================================
%
%==============================================================
global OGEOM

% All legal commands must be defined in the following table

commandtable = cell2struct({
    %--------------------------------------
    % NAME               TYPE
    %--------------------------------------
    'pointrrange',       'NNN';
    'pointrheight',      'NNN';
    'pointgeographic',   'NNN';
    'changersite',       'S';
    'format',            'S';
    'reset',             '-';
    'help',              '-';   
    'nop',               '-';
    'quit'               '-';
    % ------------------------------------
},{'name','type'},2);

% Split the command table into two lists for command name matcher

for k = 1:length(commandtable)
    cname{k} = commandtable(k).name;
    ctype{k} = commandtable(k).type;
end
OGEOM.cname = cname;
OGEOM.ctype = ctype;

return
%end define_commands


function [cmd,msg,param] = get_command(pars)
%==============================================================
% Read command from command line, return command name and
% parameters. If a valid command with correct number of
% parameters is returned, msg == '', else msg contains an
% error message.
%==============================================================
global OGEOM
msg = '';
cmd = '';
param = {};

if nargin > 0
    % Concatenate the arguments into a single string

    s = '';
    for k = 1:length(pars)
        if ischar(pars{k})
            s = [s ' ' pars{k}];
        else
            s = [s ' ' num2str(pars{k})];
        end                        
    end
else 
    % Read input from command line

    s = input('geo% ', 's');    
end    

if isempty(s) 
    cmd = 'nop';
    msg = '';
    return;
end

% First token on command line must be the command name, match it

[cmd,str] = strtok(s);
K = matcher(cmd,OGEOM.cname);

if length(K) == 0
    % No match

    cmd = '';
    msg = ['Illegal command (' cmd ')'];
    
elseif length(K) > 1
    % More than one match
    
    cmd = '';
    msg = [' cmd ' -- ambiguous'];
else
    % Unique match, now find the command parameters   
    
    str = strip(str);
    
    cmd = OGEOM.cname{K};
    if  strcmp(OGEOM.ctype{K},'-')
        str = '';
    elseif strcmp(OGEOM.ctype{K},'NNN')        
        p = str2num(str);
        if length(p) ~= 3
            msg = 'Illegal format'
        else
            param{1} = p(1);
            param{2} = p(2);
            param{3} = p(3);
        end
    elseif strcmp(OGEOM.ctype{K},'S')        
        param{1} = str;
    else
        error([mfilename ': Illegal parameter type'])
    end        
    
end
str = strip(str);

return
%get_command


function K = matcher(name, namelist)
%================================================================
%K = MATCHER(NAME, NAMELIST)
%
% Return the index of NAME in NAMELIST. The NAME can be appreviated,
% and the case is not significant. NAMELIST can be cell array
% or a character array. If no match is found, K is returned empty.
% If a unique match is found, K is the index of the NAME in NAMELIST.
% If there are several matches, K is a vector of indexes of the
% matching names. It is assumed that there are do dublicates in words
% (taking into account that case is ignored). The NAMELIST should be
% in all lowercase.
%
% In case of multiple match where one is an exact match, the
% exact match is returned, and the situation is _not_
% reported as being ambiguous.
%
% 27-Oct-1999 Jm
%================================================================
name = lower(name);
K = strmatch(name,namelist);
if length(K) >= 2
    % Got several matches. We need to check if one is an exact
    % match. It is sufficient here to check the lengths only.
    k = [];
    if iscellstr(namelist) 
        for n = 1:length(K)
            fullname = namelist{K(n)};
            if length(name) == length(fullname);
                k = K(n);
                break;
            end                
        end        
    else
        for n = 1:length(K)
            fullname = deblank(namelist(K(n),:));
            if length(name) == length(fullname);
                k = K(n);
                break;
            end                
        end        
    end
    if ~isempty(k)  % got a unique match
        K = k;
    end                        
end    
%end matcher


function str = strip(str)
%================================================================= 
% Remove leading and trailing blanks.
%=================================================================
str = str(end:-1:1);
str = deblank(str);
str = str(end:-1:1);
str = deblank(str);


function print(G)
%================================================================= 
% 
%=================================================================
global OGEOM
fprintf(1,'\n')

fprintf(1, ' Lat  = %8.3f   Lon    = %7.3f   Hgt  = %8.3f\n\n',...
            G.Lat,            G.Lon,            G.Hgt);


fprintf(1, ' azT  = %8.3f   elT    = %7.3f   rT   = %8.3f\n', ...
           G.azT,            G.elT,            G.rT);
           
fprintf(1, ' azK  = %8.3f   elK    = %7.3f   rK   = %8.3f\n', ...
           G.azK,            G.elK,            G.rK);
           
fprintf(1, ' azS  = %8.3f   elS    = %7.3f   rS   = %8.3f\n', ...
           G.azS,            G.elS,            G.rS);
           
fprintf(1, ' azE  = %8.3f   elE    = %7.3f   rE   = %8.3f\n\n',...
           G.azESR,            G.elESR,            G.rESR);

if strcmp(OGEOM.format, 'short')

    fprintf(1, ' PhaS = %6.1f     AmpS   = %5.1f\n',...
               G.PhaS,            G.AmpS );

    fprintf(1, ' PhaK = %6.1f     AmpK   = %5.1f\n',...
               G.PhaK,            G.AmpK );

end

if strcmp(OGEOM.format, 'long')

    fprintf(1, ' PhaS = %6.1f     AmpS   = %5.1f     PoFS = %8.3f\n',...
               G.PhaS,            G.AmpS,            G.PolfS);

    fprintf(1, ' PhaK = %6.1f     AmpK   = %5.1f     PoFK = %8.3f\n\n',...
               G.PhaK,            G.AmpK,            G.PolfK);

    %fprintf(1, ' PhaE = %8.1f   AmpE   = %7.1f   PoFE = %8.3f\n\n',...
    %           G.PhaESR,            G.AmpESR,            G.PolfESR);

    fprintf(1, ' gamS = %6.1f     gamK   = %5.1f     gamE = %6.1f\n',...
                G.gammaS,            G.gammaK,              G.gammaESR);

    fprintf(1, ' volS = %6.1f     delayS = %5.1f\n' ,G.volS,   G.delayS)
    fprintf(1, ' volK = %6.1f     delayK = %5.1f\n' ,G.volK,   G.delayK)
    %fprintf(1, ' volE = %8.1f   delayE = %7.1f\n' ,G.volESR, G.delayESR)
end 

fprintf(1,'\n')

return
%end print
