function varargout = setpolariser(G)

%SETPOLARISER(G)
%
%   Picks the polarizer settings for KIRUNA and SODANKYLA
%   from the structure G, which is the output structure of the
%   pointing geometry commands like pointrrange.
%
%   Without input argument, the "G"  corresponding to the latest
%   pointing command is used.
%
%   Without an output argument, SETPOLARISER prints the
%   polarizer settings in a format that can directly be copied
%   to the experiment's .elan file.
%
%   Example:
%
%           >> G = pointrrange(180,77,300,'T');
%
%           >> setpolariser(G)    ===>
%
%           if [ISKIR] {setpolariser 267 -7}
%           if [ISSOD] {setpolariser 254 -18}
%
%           >> P = setpolariser  ===>
%           P = 
%           PhaK: 267
%           AmpK: -7
%           PhaS: 254
%           AmpS: -18
%
%   24-Mar-2002 Jm

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
if nargin == 0 
    [G,msg] = ogeom_setG;
    if isempty(G)
        fprintf(1,'%s\n',msg)
        return
    end                
elseif nargin > 1
    error('Wrong number of input arguments')    
end    

if isfield(G,'PhaS')
    P.PhaK = round(G.PhaK);
else
    error('Invalid input')
end

P.AmpK = round(G.AmpK);
P.PhaS = round(G.PhaS);
P.AmpS = round(G.AmpS);

if nargout == 0
    fprintf(1,'\nif [ISKIR] {setpolariser %d %d 1}\n', P.PhaK, P.AmpK);        
    fprintf(1,'if [ISSOD] {setpolariser %d %d 1}\n\n', P.PhaS, P.AmpS);        
elseif nargout == 1
    varargout{1} = P;
end    
        

